@@ -64,7 +64,10 @@ int main(int argc, char *argv[])
 	fd_set rsock;
 	pid_t newpid;
 	struct timeval tv;
-
+#ifdef NEW_SIGNALS
+        struct sigaction act;
+	struct sigaction old;
+#endif
 	do_console = 0;
 	do_debug = OER_DEBUG_INFO;
 	do_help = 0;
@@ -129,6 +132,27 @@ int main(int argc, char *argv[])
 		close(STDERR_FILENO);
 		do_console = 0;
 	}
+#ifdef NEW_SIGNALS
+	sigemptyset(&act.sa_mask);
+        act.sa_flags = SA_RESTART;
+        act.sa_handler = handlesignal;
+        if(sigaction(SIGHUP, &act, &old) < 0) {
+                oer_debug(OER_DEBUG_INFO, "main->failed to set SIGHUP: %s\n", strerror(errno));
+                exit(EXIT_FAILURE);
+	}
+        if(sigaction(SIGHUP, &act, &old) < 0) {
+                oer_debug(OER_DEBUG_INFO, "main->failed to set SIGHUP: %s\n", strerror(errno));
+                exit(EXIT_FAILURE);
+	}
+        if(sigaction(SIGHUP, &act, &old) < 0) {
+                oer_debug(OER_DEBUG_INFO, "main->failed to set SIGHUP: %s\n", strerror(errno));
+                exit(EXIT_FAILURE);
+	}
+        if(sigaction(SIGHUP, &act, &old) < 0) {
+                oer_debug(OER_DEBUG_INFO, "main->failed to set SIGHUP: %s\n", strerror(errno));
+                exit(EXIT_FAILURE);
+	}
+#else
 	if(signal(SIGHUP, handlesignal) == SIG_ERR) {
                 oer_debug(OER_DEBUG_INFO, "main->failed to set SIGHUP: %s\n", strerror(errno));
                 exit(EXIT_FAILURE);
@@ -145,6 +169,7 @@ int main(int argc, char *argv[])
                 oer_debug(OER_DEBUG_INFO, "main->failed to set SIGPIPE: %s\n", strerror(errno));
                 exit(EXIT_FAILURE);
         }
+#endif
 	if((mystate = emptystate()) == NULL) {
 		oer_debug(OER_DEBUG_INFO, "main->failed to allocate memory for irc state\n");
 		exit(EXIT_FAILURE);
